#include <iostream>
#include "polinom.h"

using namespace std;

// Konstruktor a fokszm megadsval.
Polinom::Polinom(int fokszam) throw (Kivetelek)
{
    if(fokszam < 0)
        throw HibasFokszam;

    this->fokszam = fokszam;
    egyutthatok.resize(fokszam);
    for(int i = 0; i < fokszam; ++i)
        egyutthatok[i] = 0; // kezdetben miden egytthat legyen 0.
}

// Belltja a megadott kitevj egytthat rtkt.
void Polinom::beallit(int kitevo, int ertek) throw (Kivetelek)
{
    if(kitevo >= 0 && kitevo < fokszam)
        egyutthatok[kitevo] = ertek;
    else
        throw HibasKitevo;
}

// Polinomok sszeadsa
Polinom Polinom::operator+(const Polinom &masik)
{
    // Elszr meghatrozzuk a nagyobbik fokszmot.
    int nagyobbFokszam;
    if(fokszam > masik.fokszam)
        nagyobbFokszam = fokszam;
    else
        nagyobbFokszam = masik.fokszam;

    // Majd egy sszegpolinomba pronknt sszeadjuk a kitevket.
    Polinom osszeg(nagyobbFokszam);
    for(int i = 0; i < nagyobbFokszam; ++i)
    {
        // Ellenrzssel gyelnk arra, hogy a kisebbik fokszm polinomon nehogy "tlfussunk" s nem ltez egytthatkat keressnk.
        int iEgyutthato = 0;
        if(fokszam > i)
            iEgyutthato += egyutthatok[i];
        if(masik.fokszam > i)
            iEgyutthato += masik.egyutthatok[i];
        osszeg.beallit(i, iEgyutthato);
    }

    return osszeg;
}

// Polinomok szorzsa
Polinom Polinom::operator*(const Polinom &masik)
{
    // A szorzatpolinom fokszma a kt operandus fokszmnak sszege.
    Polinom szorzat(fokszam + masik.fokszam);

    // Dupla ciklusban minden tagot megszorozzuk mindegyikkel a msik polinombl.
    for(int i = 0; i < fokszam; ++i)
        for(int j = 0; j < masik.fokszam; ++j)
        {
            int kitevo = i + j; // Kt tag sszeszorzsakor az eredmny kitevje az operendusok kitevinek sszege.
            // Mivel a szorzatban egy tag egytthatjt sszegknt kapjuk meg, ezrt nem fellrjuk, hanem hozzadjuk a meglv egytthathoz az jabb rtket.
            szorzat.beallit(kitevo, szorzat.egyutthatok[kitevo] + egyutthatok[i] * masik.egyutthatok[j]);
        }

    return szorzat;
}

// Beolvas opertor
istream& operator>>(istream& be, Polinom &p)
{
    for(int i = p.fokszam - 1; i >= 0; --i)
        be >> p.egyutthatok[i];
    return be;
}

// Kir opertor
ostream& operator<<(ostream& ki, Polinom &p)
{
    for(int i = p.fokszam - 1; i >= 0; --i)
    {
        ki << p.egyutthatok[i] << "x^" << i;
        if(i > 0)
            ki << " + ";
    }
    ki << endl;
    return ki;
}
